#include <stdio.h>
#include <string.h>
#include "gg.h"

/* maps a letter to binary value. Returns -1 if
 * letter is invalid */
static int getlettervalue(char letter)
{
	switch (letter)
	{
		case 'a':
		case 'A': return 0x0;
		case 'p':
		case 'P': return 0x1;
		case 'z':
		case 'Z': return 0x2;
		case 'l':
		case 'L': return 0x3;
		case 'g':
		case 'G': return 0x4;
		case 'i':
		case 'I': return 0x5;
		case 't':
		case 'T': return 0x6;
		case 'y':
		case 'Y': return 0x7;
		case 'e':
		case 'E': return 0x8;
		case 'o':
		case 'O': return 0x9;
		case 'x':
		case 'X': return 0xA;
		case 'u':
		case 'U': return 0xB;
		case 'k':
		case 'K': return 0xC;
		case 's':
		case 'S': return 0xD;
		case 'v':
		case 'V': return 0xE;
		case 'n':
		case 'N': return 0xF;
	}
	return -1;
}


int gg_decode6letters(char *code, unsigned short *address, unsigned char *val)
{
	int i, value;
	unsigned char bcode[6];

	if (!code || !address || !val) { return -1; }
	if (strlen(code) != 6) { return -1; }
	
	for (i=0; i<6; i++)
	{
		value = getlettervalue(code[i]);
		if (value<0) { 
			fprintf(stderr, "invalid letter\n");
			return -1;
		}
		bcode[i] = value;
	}

#if 0
	printf("binary code: ");
	for (i=0; i<6; i++)
	{
		int j;
		for (j=8; j; j>>=1)
		{
			if (bcode[i]&j) { 
				printf("1"); 
			} else {
				printf("0");
			}
		}
		printf(" ");
	}
	printf("\n");
#endif

	*val = 0;
	*val |= (bcode[0] & 8) ? 0x80 : 0x00; // 1
	*val |= (bcode[1] & 4) ? 0x40 : 0x00; // 2
	*val |= (bcode[1] & 2) ? 0x20 : 0x00; // 3
	*val |= (bcode[1] & 1) ? 0x10 : 0x00; // 4
	*val |= (bcode[5] & 8) ? 0x08 : 0x00; // 5
	*val |= (bcode[0] & 4) ? 0x04 : 0x00; // 6
	*val |= (bcode[0] & 2) ? 0x02 : 0x00; // 7
	*val |= (bcode[0] & 1) ? 0x01 : 0x00; // 8

	*address = 0;
	*address |= (bcode[3] & 4) ? 0x8000 : 0x0000; // a
	*address |= (bcode[3] & 2) ? 0x4000 : 0x0000; // b
	*address |= (bcode[3] & 1) ? 0x2000 : 0x0000; // c
	*address |= (bcode[4] & 8) ? 0x1000 : 0x0000; // d
	*address |= (bcode[5] & 4) ? 0x0800 : 0x0000; // e
	*address |= (bcode[5] & 2) ? 0x0400 : 0x0000; // f
	*address |= (bcode[5] & 1) ? 0x0200 : 0x0000; // g
	*address |= (bcode[1] & 8) ? 0x0100 : 0x0000; // h
	*address |= (bcode[2] & 4) ? 0x0080 : 0x0000; // i
	*address |= (bcode[2] & 2) ? 0x0040 : 0x0000; // j
	*address |= (bcode[2] & 1) ? 0x0020 : 0x0000; // k
	*address |= (bcode[3] & 8) ? 0x0010 : 0x0000; // l
	*address |= (bcode[4] & 4) ? 0x0008 : 0x0000; // m
	*address |= (bcode[4] & 2) ? 0x0004 : 0x0000; // n
	*address |= (bcode[4] & 1) ? 0x0002 : 0x0000; // o
	
	return 0;
}

int gg_decode8letters(char *code, unsigned short *address, unsigned char *val, unsigned char *cval)
{
	int i, value;
	unsigned char bcode[8];

	if (!code || !address || !val || !cval) { return -1; }
	if (strlen(code) != 8) { return -1; }
	
	for (i=0; i<8; i++)
	{
		value = getlettervalue(code[i]);
		if (value<0) { 
			fprintf(stderr, "invalid letter\n");
			return -1;
		}
		bcode[i] = value;
	}

#if 0
	printf("binary code: ");
	for (i=0; i<6; i++)
	{
		int j;
		for (j=8; j; j>>=1)
		{
			if (bcode[i]&j) { 
				printf("1"); 
			} else {
				printf("0");
			}
		}
		printf(" ");
	}
	printf("\n");
#endif

	*val = 0;
	*val |= (bcode[0] & 8) ? 0x80 : 0x00; // 1
	*val |= (bcode[1] & 4) ? 0x40 : 0x00; // 2
	*val |= (bcode[1] & 2) ? 0x20 : 0x00; // 3
	*val |= (bcode[1] & 1) ? 0x10 : 0x00; // 4
	*val |= (bcode[7] & 8) ? 0x08 : 0x00; // 5
	*val |= (bcode[0] & 4) ? 0x04 : 0x00; // 6
	*val |= (bcode[0] & 2) ? 0x02 : 0x00; // 7
	*val |= (bcode[0] & 1) ? 0x01 : 0x00; // 8

	*address = 0;
	*address |= (bcode[3] & 4) ? 0x8000 : 0x0000; // a
	*address |= (bcode[3] & 2) ? 0x4000 : 0x0000; // b
	*address |= (bcode[3] & 1) ? 0x2000 : 0x0000; // c
	*address |= (bcode[4] & 8) ? 0x1000 : 0x0000; // d
	*address |= (bcode[5] & 4) ? 0x0800 : 0x0000; // e
	*address |= (bcode[5] & 2) ? 0x0400 : 0x0000; // f
	*address |= (bcode[5] & 1) ? 0x0200 : 0x0000; // g
	*address |= (bcode[1] & 8) ? 0x0100 : 0x0000; // h
	*address |= (bcode[2] & 4) ? 0x0080 : 0x0000; // i
	*address |= (bcode[2] & 2) ? 0x0040 : 0x0000; // j
	*address |= (bcode[2] & 1) ? 0x0020 : 0x0000; // k
	*address |= (bcode[3] & 8) ? 0x0010 : 0x0000; // l
	*address |= (bcode[4] & 4) ? 0x0008 : 0x0000; // m
	*address |= (bcode[4] & 2) ? 0x0004 : 0x0000; // n
	*address |= (bcode[4] & 1) ? 0x0002 : 0x0000; // o

	*cval = 0;
	*cval |= (bcode[6] & 8) ? 0x80 : 0x00; // !
	*cval |= (bcode[7] & 4) ? 0x40 : 0x00; // @
	*cval |= (bcode[7] & 2) ? 0x20 : 0x00; // #
	*cval |= (bcode[7] & 1) ? 0x10 : 0x00; // $
	*cval |= (bcode[5] & 8) ? 0x08 : 0x00; // %
	*cval |= (bcode[6] & 4) ? 0x04 : 0x00; // ^
	*cval |= (bcode[6] & 2) ? 0x02 : 0x00; // &
	*cval |= (bcode[6] & 1) ? 0x01 : 0x00; // *
	
	return 0;
}



