#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static unsigned char *prg_data;
static int prg_length;
static unsigned char *chr_data;
static int chr_length;

int readnes(FILE *fptr)
{
	unsigned char magic[4];
	unsigned char prg, chr, mapper, mapper1;
	unsigned char reserved[8];
	unsigned long filelength;
	unsigned long rom_end;
	unsigned char tmp;
	int i;

	fseek(fptr, 0, SEEK_END);
	filelength = ftell(fptr);
	fseek(fptr, 0, SEEK_SET);

	printf("File length :%ld Bytes\n", filelength);
	
	fread(magic, 4, 1, fptr);

	if (!(magic[0] == 'N' &&
			magic[1] == 'E' &&
			magic[2] == 'S' &&
			magic[3] == 0x1a))
	{
		fprintf(stderr, "Bad header\n");
		return -1;
	}
	fread(&prg, 1, 1, fptr);
	fread(&chr, 1, 1, fptr);
	fread(&mapper, 1, 1, fptr);
	fread(&mapper1, 1, 1, fptr);
	fread(reserved, 8, 1, fptr);
	
	printf("NES PRG: %02X CHR: %02X MAPPER: %02X %02X\n",
			prg,chr,mapper,mapper1);

	// incomplete
	switch(mapper>>4)
	{
		case 0: printf("No mapper\n"); break;
		case 1: printf("Nintendo MMC1\n"); break;
		case 2: printf("PRG Switch (konami)\n"); break;
		case 3: printf("VROM (CHR) Switch\n"); break;
		case 4: printf("Nintendo MMC3\n"); break;
		case 5: printf("Nintendo MMC5\n"); break;
		default:
		printf("Mapper number: %d\n", mapper>>4);
		
	}
	printf("Flags: %02x\n", mapper & 0x0f);
	switch(mapper & 0xf)
	{
		case 0: printf("H\n"); break;
		case 1: printf("V\n"); break;
		case 2: printf("H + Bat\n"); break;
		case 3: printf("V + Bat\n"); break;
	}

	prg_length = prg*16*1024;
	chr_length = chr*8*1024;
	printf("PRG %d pages of 16kb (%d bytes) \n", prg, prg_length);
	printf("CHR %d pages of 8kb (%d bytes)\n", chr, chr_length);
	
	prg_data = malloc(prg*16*1024);
	if (prg_data == NULL) { perror("malloc"); return -1; }
	chr_data = malloc(chr*8*1024);
	if (chr_data == NULL) { perror("malloc"); return -1; }

	if (fread(prg_data, prg*16*1024, 1, fptr)<0) {
		perror("fread");
		return -1;
	}
	if (fread(chr_data, chr*8*1024, 1, fptr)<0) {
		perror("fread");
		return -1;
	}

	rom_end = ftell(fptr);
	printf("End at %ld\n", rom_end);

	printf("Remaining bytes: %ld\n", filelength - rom_end);

	for (i=0; i<filelength-rom_end; i++)
	{
		fread(&tmp, 1, 1, fptr);
		printf("%c", tmp);
	}
	printf("\n");

	return 0;
}

int main(int argc, char **argv)
{
	char *input_filename;
	char *base_filename;
	char *out_filename;
	char *tmp;
	FILE *in_fptr, *out_fptr;
	int res;
	int repeat_prg=0, repeat_chr=0;

	if (argc< 2) { 
		printf("Usage: ./readnes file <num repeat prg> <num repeat chr>\n");
		return 0; 
	}
	input_filename = argv[1];
	if (argc>=3) {
		repeat_prg = atoi(argv[2]);
	}
	if (argc>=4) { 
		repeat_chr = atoi(argv[3]);
	}

	in_fptr = fopen(input_filename, "rb");
	if (!in_fptr) { perror("fopen"); return 1; }

	res = readnes(in_fptr);
	if (res<0) {
		fclose(in_fptr);
		return 1;
	}
	fclose(in_fptr);
	
	base_filename = strdup(input_filename);

	// remove everything after the last dot
	tmp = strrchr(base_filename, '.');
	if (tmp) { *tmp = 0; }

	// allocate base filename, .PRG or CHR and NUL
	out_filename = malloc(strlen(base_filename) + 4 + 1);	

	sprintf(out_filename, "%s.prg", base_filename);
	out_fptr = fopen(out_filename, "w");	
	if (!out_fptr) { perror(out_filename); goto cleanup; } 
	printf("Writing %s\n", out_filename);
	
	do {
		printf(".");
		fwrite(prg_data, prg_length, 1, out_fptr);
	} while(--repeat_prg > 0);
	printf("\n");
	
	fclose(out_fptr);

	sprintf(out_filename, "%s.chr", base_filename);
	out_fptr = fopen(out_filename, "w");	
	if (!out_fptr) { perror(out_filename); goto cleanup; } 
	printf("Writing %s\n", out_filename);
	do {
		printf(".");
		fwrite(chr_data, chr_length, 1, out_fptr);
	} while(--repeat_chr > 0);
	printf("\n");


cleanup:
	fclose(out_fptr);
	free(base_filename);
	free(out_filename);
	
	return 0;
}
